#include <bits/stdc++.h>
using namespace std;

ifstream in("input.txt");
ofstream out("output.txt");

vector<vector<pair<int,int>>> graph;
vector<vector<int>> tree;
vector<int> dist;
int n, m, s;

// struttura per dijkstra
struct node {
    int id; // nodo corrente
    int dist; // distanza dalla sorgente fino a questo nodo
    int parent; // nodo da cui sono arrivato
};

bool operator < (const node a, const node b){
    return a.dist > b.dist; // per avere una min priority queue
}

// dijkstra che costruisce il dag dei camminimi minimi (un albero in questo caso)
void dijkstra(int source, vector<bool>& visited){ 
    priority_queue<node> q; 
    q.push({source,0,-1}); 
    dist[source] = 0; 
  
    while (!q.empty()){ 
        int u = q.top().id; 
        int d = q.top().dist;
        int prec = q.top().parent;
        q.pop(); 
        if(dist[u] == d && prec != -1){
            tree[prec].push_back(u);
        }
        if(!visited[u]){ 
            visited[u] = true;
            for(pair<int,int> p : graph[u]) { 
                int v = p.first; 
                int weight = p.second; 
                if(!visited[v]){
                    if (dist[v] >= dist[u] + weight){ 
                        dist[v] = dist[u] + weight; 
                        q.push({v,dist[v],u}); 
                    } 
                }
            } 
        }
    } 
} 

// root: radice dell'albero, s: figlio della radice e punto di partenza per la visita
// mybfs: bfs che non passa per root (quindi visita il sottoalbero radicato nel nodo s)
// e conta i nodi raggiungibili da s
// print: flag, se true viene stampato l'elenco di questi nodi
int mybfs(int s, int root, vector<bool>& visited, bool print){
    queue<int> q;
    q.push(s);
    visited[s] = true;
    int c = 1; 
    while(!q.empty()) {
        int v = q.front();
        q.pop();
        if(print) out << v << endl;
        for(int i = 0; i<tree[v].size(); i++){
            int u = tree[v][i];
            if(!visited[u] && u != root){
                visited[u] = true;
                c++;
                q.push(u);
            }
        }
    }
    return c;
}

int main(){
    in >> n >> m >> s;

    graph.resize(n);
    tree.resize(n);
    dist.resize(n, INT_MAX); 

    for(int i = 0; i<m; i++){
        int u, v, w;
        in >> u >> v >> w;
        graph[u].push_back(make_pair(v, w));
        graph[v].push_back(make_pair(u, w));
    }

    {
        vector<bool> visited(n+1, false);
        dijkstra(s, visited);
    }

    // sol caso alberi, applicata all'albero dei cammini minimi 
    int res = 0, node;
    for(int i = 0; i<tree[s].size(); i++){
        int v = tree[s][i];
        vector<bool> visited(n+1, false);
        int curr = mybfs(v, s, visited, false);
        if(curr > res){
            res = curr;
            node = v;
        }
    }

    out << res << endl;

    vector<bool> visited(n, false);
    mybfs(node, s, visited, true);

    return 0;
}
